import os
import re
import json

import logging
logging.basicConfig(level=logging.INFO)

from pythonping import ping
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from Onvif_proxy.apps.onvif_proccesser.Make_request.make_request import make_request

from Utils.utils import go_up

#python onvif-proxy/django-app/manage.py runcustomserver 192.168.1.86 80 "path" "1" "path"

servicepatterns = {
    'device': ['device', 'device_service', 'services'],
    'services': ['device', 'device_service', 'services'],
    'analytics': ['analytics_service', 'Analytics', 'analytics'],
    'event': ['event_service', 'event', 'Events', 'Event'],
    'events': ['event_service', 'event', 'Events', 'Event'],
    'image': ['imaging_service', 'image_service', 'Imaging', 'imaging'],
    'imaging': ['imaging_service', 'image_service', 'Imaging', 'imaging'],
    'deviceio': ['deviceIO_service', 'deviceio_service', 'deviceio'],
    'media': ['media_service', 'Media', 'media'],
    'ptz': ['ptz', 'ptz_service', 'PTZ'],
    'onvif_ext': ['onvif_ext'],
    'Subscription': ['Subscription'],
    'scdl_service': ['scdl_service']
}

cameras = json.load(open(os.path.join(os.path.dirname(__file__), 'conf.json')))

csettings = json.load(open(os.path.join(go_up(os.path.dirname(__file__), 3), 'start_conf.json')))
print(csettings)
try:
    os.remove(os.path.join(go_up(os.path.dirname(__file__), 3), 'start_conf.json'))

def get_replace_template(ipport, host_ip, octet = None, port = None):
    if port == None and octet == None:
        to_replace = r'http://{}/'.format(host_ip, octet)
    elif port == None:
        to_replace = r'http://{}/{}/'.format(host_ip, octet)
    else:
        to_replace = r'http://{}:{}/{}/'.format(host_ip, port, octet)
    dict = {
        r'{}'.format(ipport): to_replace,
        r'http://255.255.255.255:80/': to_replace
    }
    return dict

def replace_all(text, dic):
    for i, j in dic.items():
        text = text.replace(i, j)
    return text

def return_soap(response):
    return HttpResponse(response, headers={
        'Content-Type': 'application/soap+xml'
    })

def get_response(service, Onvif, body, octet = None):
    if len(service) <= 30:
        check_connection = str(ping(Onvif.ip, timeout = 2, count = 1))
        if check_connection.find('Request timed out') == -1:
            response = Onvif.make_request(service, body)
            if response != '':
                if response.status_code != 500:
                    if response.status_code == 400 or response.text == '' or response.text.find('400 Bad Request') > 0 or response.text.find('404 Not Found') > 0 or response.text.find('404 File Not Found') > 0:
                        service = service.split('_')[0].lower()
                        if service.rfind('/') > -1:
                            to_list = service.split('/')
                            service = to_list[len(to_list) - 1].lower()
                        if service in list(servicepatterns.keys()):
                            service_length = len(servicepatterns[service])
                            iterrator = 0
                            while (response.status_code == 400 or response.text == '' or response.text.find('400 Bad Request') > 0 or response.text.find('404 Not Found') > 0 or response.text.find('404 File Not Found') > 0) and iterrator < service_length:
                                response = Onvif.make_request(servicepatterns[service][iterrator], body)
                                iterrator += 1
                        else:
                            logging.info('Нет сервиса {}'.format(service))
                    if response.status_code == 400 or response.text == '' or response.text.find('400 Bad Request') > 0 or response.text.find('404 Not Found') > 0 or response.text.find('404 File Not Found') > 0:
                        ()
                        return ''
                    else:
                        return response.text
        else:
            return ''

def replace_token(response, octet):
    if response == None:
        return ''
    else:
        token_as = [m.start() for m in re.finditer('token=', response)]
        token_arrow = [m.start() for m in re.finditer('Token>', response)]
        token_to_replace_as = []
        token_to_replace_arrow = []
        for i in token_as:
            token = response[i + 7:]
            token = token[: token.find('"')]
            token_to_replace_as.append(token)
        for i in range(len(token_arrow)):
            if i % 2 == 1:
                pass
            else:
                token = response[token_arrow[i] + 5:]
                token = token[: token.find('<')]
                if '<' in token:
                    token = token.replace('<', '')
                elif '>' in token:
                    token = token.replace('>', '')
                token_to_replace_arrow.append(token)
        token_to_replace_as = [*set(token_to_replace_as)]
        token_to_replace_arrow = [*set(token_to_replace_arrow)]
        split = response.find('Body') + 5
        first_part = response[:split]
        second_part = response[split:]
        response = first_part
        if len(token_to_replace_as) > 0:
            for what_replace in token_to_replace_as:
                if what_replace != '':
                    token_what_replace = 'token="' + what_replace + '"'
                    to_replace = 'token="' + octet + '_' + what_replace + '"'
                    second_part = second_part.replace(token_what_replace, to_replace)
        if len(token_to_replace_arrow) > 0:
            for what_replace in token_to_replace_arrow:
                if what_replace != '':
                    token_what_replace = 'Token>' + what_replace + '<'
                    to_replace = 'Token>' + octet + '_' + what_replace + '<'
                    second_part = second_part.replace(token_what_replace, to_replace)
        response = response + second_part
        return response

@csrf_exempt
def image(request, picture):
    path = os.path.join(os.path.dirname(__file__), 'picture', picture)
    with open(path, "rb") as f:
        return HttpResponse(f.read(), content_type="image/jpeg")

@csrf_exempt
def index(request, number):
    try:
        if int(number) < 257:
            return camera(request, number)
        else:
            return auditory(request, number)
    except ValueError:
        return auditory(request, number)

@csrf_exempt
def without_number(request):
    try:
        if int(cameras['default']) < 256:
            return camera(request, cameras['default'])
        else:
            return remote_controller(request, cameras['default'], 'remote_controller', cameras["default_cam"])
    except ValueError:
        return remote_controller(request, cameras['default'], 'remote_controller', cameras["default_cam"])

def camera(request, number):

    logging.info('Камера {}'.format(number))

    cam = str(number)
    ip = cameras['cameras'][cam]['ip']

    port = cameras['cameras'][cam]['port']
    ipport = 'http://{}:{}/'.format(ip, port)
    streamuri = cameras['cameras'][cam]['streamuri']

    header = str(request.headers)
    action = header[header.find('action') + 8: ]
    action = action[:action.find('"')]
    service = action[action.find('ver10') + 6 : action.find('wsdl') - 1]

    body = request.body.decode()

    Onvif = make_request(ip, port)

    if body.find('GetCapabilities') > 0:

        response = Onvif.make_request('device_service', body)

        response = replace_all(response.text, get_replace_template(ipport, cameras['host_ip'], cam, cameras['port']))
        response = replace_all(response, get_replace_template(r'http://{}/'.format(ip), cameras['host_ip'], cam, None))
        response = response.encode()

    else:
        if service == 'device':
            response = Onvif.make_request('device_service', body).text
        else:
            response = Onvif.make_request(service, body).text

        if body.find('GetStreamUri') > 0:

            old_streamuri = response[response.find('>rtsp') + 1 : ]
            old_streamuri = old_streamuri[ : old_streamuri.find('<')]
            response = response.replace(old_streamuri, streamuri)
        if body.find('GetScopes') > 0:
            if response.find('/location/') and response.find('/location/') != response.find('/location/country'):
                response = response.replace('/location/', '/location/{}'.format(cam))
            elif response.find(':location:'):
                location = response[response.find(':location:') + 10:]
                location = location[:location.find('<')]

                response = response.replace(location, '{}'.format(cam))

    return return_soap(response)

def auditory(request, number):

    logging.info('Аудитория {}'.format(number))

    auditory = str(number)
    ips = list(cameras['auditories'][auditory].keys())
    ip = cameras['auditories'][auditory][ips[0]]['ip']

    port = cameras['auditories'][auditory][ips[0]]['port']
    ipport = 'http://{}:{}/'.format(ip, port)

    body = request.body.decode()

    Onvif = make_request(ip, port)

    if body.find('GetCapabilities') > 0 or body.find('GetServices') > 0:
        response = get_response('device_service', Onvif, body)
        response = replace_all(response, get_replace_template(ipport, cameras['host_ip'], auditory, cameras['port']))
        response = replace_all(response, get_replace_template(r'http://{}/'.format(ip), cameras['host_ip'], auditory, cameras['port']))
        response = response.encode()
    else:
        header = str(request.headers)
        logging.info('Хедер {}'.format(header))
        action = header[header.find('action') + 8:]
        action = action[:action.find('"')]
        service = action[action.find('ver10') + 6: action.find('wsdl') - 1]
        logging.info('Сервис {}'.format(service))

        if body.find('GetScopes') > 0:
            response = get_response(service, Onvif, body)
            logging.info('До {}'.format(response))

            if response.find('/location/') > 0 and response.find('/location/') != response.find('/location/country'):
                response = response.replace('/location/', '/location/{}'.format(auditory))
            elif response.find(':location:') > 0:
                location = response[response.find(':location:') + 10:]
                location = location[:location.find('<')]
                response = response.replace(location, '{}'.format(auditory))
            elif response.find('/location/') > 0:
                location = response[response.find('/location/') + 10:]
                location = location[:location.find('<')]
                response = response.replace(location, '{}'.format(auditory))
            logging.info('После {}'.format(response))

        elif body.find('GetProfiles') > 0:
            response = get_response(service, Onvif, body)
            response = replace_token(response, ips[0])
            first_part = response[:response.rfind('GetProfilesResponse')]
            first_part = first_part[:first_part.rfind('>') + 1]
            second_part = response[response.rfind('Profiles>'):]
            second_part = second_part[second_part.find('<'):]
            response = first_part
            for i in range(1, len(ips)):
                Onvif = make_request(cameras['auditories'][auditory][ips[i]]['ip'], cameras['auditories'][auditory][ips[i]]['port'])
                add_response = get_response(service, Onvif, body, ips[i])
                add_response = replace_token(add_response, ips[i])
                add_response = add_response[add_response.find('GetProfilesResponse') + 20:add_response.rfind('GetProfilesResponse')]
                add_response = add_response[:add_response.rfind('>') + 1]
                response = response + add_response
            response = response + second_part
        elif body.find('GetVideoSources') > 0:
            response = get_response(service, Onvif, body)
            response = replace_token(response, ips[0])
            first_part = response[:response.rfind('GetVideoSourcesResponse')]
            first_part = first_part[:first_part.rfind('>') + 1]
            second_part = response[response.rfind('VideoSources>'):]
            second_part = second_part[second_part.find('<'):]
            response = first_part
            for i in range(1, len(ips)):
                Onvif = make_request(cameras['auditories'][auditory][ips[i]]['ip'], cameras['auditories'][auditory][ips[i]]['port'])

                add_response = get_response(service, Onvif, body, ips[i])

                add_response = replace_token(add_response, ips[i])
                add_response = add_response[add_response.find('GetVideoSourcesResponse') + 24:add_response.rfind('GetVideoSourcesResponse')]
                add_response = add_response[:add_response.rfind('>') + 1]
                response = response + add_response
            response = response + second_part
            response = re.sub("^\s+|\n|\r|\s+$", '', response)
        elif (body.rfind('Token') - 8) != body.rfind('UsernameToken') and body.rfind('Token') > -1 and body.rfind('UsernameToken') > -1:
            token_with_ip = body[body.rfind('Token>', 0, body.rfind('Token>') - 7) + 6:]
            token_with_ip = token_with_ip[: token_with_ip.find('<')]

            octet = token_with_ip.split('_')[0]

            token_without_ip = token_with_ip[len(octet) + 1:]

            body = body.replace(token_with_ip, token_without_ip)
            if body.find('GetVideoEncoderConfigurationOptions') > 0 or body.find('GetMetadataConfigurationOptions') > 0:
                if body.find('ConfigurationToken>') > 0:
                    token_with_ip = body[body.find('ConfigurationToken>') + 19:]
                    token_with_ip = token_with_ip[: token_with_ip.find('<')]
                    token_without_ip = token_with_ip[len(octet) + 1:]
                    body = body.replace(token_with_ip, token_without_ip)

            Onvif = make_request(cameras['auditories'][auditory][octet]['ip'], cameras['auditories'][auditory][octet]['port'])

            response = get_response(service, Onvif, body)

            if body.find('GetStreamUri') > 0:
                old_streamuri = response[response.find('>rtsp') + 1:]
                old_streamuri = old_streamuri[: old_streamuri.find('<')]
                streamuri = cameras['auditories'][auditory][octet]['streamuri']
                response = response.replace(old_streamuri, streamuri)
            response = replace_token(response, octet)

            if body.find('GetSnapshotUri'):
                if response.find('255.255.255.255:80'):
                    response = replace_all(response, get_replace_template(ipport, cameras['auditories'][auditory][octet]['ip']))
        else:
            response = get_response(service, Onvif, body)
            response = replace_token(response, ips[0])
        response = re.sub("^\s+|\n|\r|\s+$", '', response)

    return return_soap(response)

def remote_controller(request, number, paradigm, cam_num = -1):

    logging.info('Аудитория {}'.format(number))
    logging.info('Камера {}'.format(cam_num))

    auditory = str(number)

    ips = list(cameras[paradigm][auditory].keys())
    if int(cam_num) > -1:
        needed_ip = cam_num
    else:
        needed_ip = ips[0]

    ip = cameras[paradigm][auditory][needed_ip]['ip']
    port = cameras[paradigm][auditory][needed_ip]['port']

    #ip = settings.start_ip
    #port = settings.start_port

    ipport = 'http://{}:{}/'.format(ip, port)

    body = request.body.decode()

    Onvif = make_request(ip, port)

    if body.find('GetCapabilities') > 0 or body.find('GetServices') > 0:
        response = get_response('device_service', Onvif, body)
        response = replace_all(response, get_replace_template(ipport, cameras['host_ip']))#, auditory, cameras['port']
        response = replace_all(response, get_replace_template(r'http://{}/'.format(ip), cameras['host_ip']))#, auditory, cameras['port']
        response = response.encode()
    else:
        header = str(request.headers)
        logging.info('Хедер {}'.format(header))
        action = header[header.find('action') + 8:]
        action = action[:action.find('"')]
        service = action[action.find('ver10') + 6: action.find('wsdl') - 1]
        logging.info('Сервис {}'.format(service))

        if body.find('GetScopes') > 0:
            response = get_response(service, Onvif, body)

            if response.find('/location/') > 0 and response.find('/location/') != response.find('/location/country'):
                response = response.replace('/location/', '/location/{}'.format(auditory))
            elif response.find(':location:') > 0:
                location = response[response.find(':location:') + 10:]
                location = location[:location.find('<')]
                response = response.replace(location, '{}'.format(auditory))
            elif response.find('/location/') > 0:
                location = response[response.find('/location/') + 10:]
                location = location[:location.find('<')]
                response = response.replace(location, '{}'.format(auditory))

        elif body.find('GetProfiles') > 0:
            response = get_response(service, Onvif, body)
            response = replace_token(response, needed_ip)
            if int(cam_num) < 0:
                first_part = response[:response.rfind('GetProfilesResponse')]
                first_part = first_part[:first_part.rfind('>') + 1]
                second_part = response[response.rfind('Profiles>'):]
                second_part = second_part[second_part.find('<'):]
                response = first_part
                for i in range(1, len(ips)):
                    Onvif = make_request(cameras[paradigm][auditory][ips[i]]['ip'], cameras[paradigm][auditory][ips[i]]['port'])
                    add_response = get_response(service, Onvif, body, ips[i])
                    add_response = replace_token(add_response, ips[i])
                    add_response = add_response[add_response.find('GetProfilesResponse') + 20:add_response.rfind('GetProfilesResponse')]
                    add_response = add_response[:add_response.rfind('>') + 1]
                    response = response + add_response
                response = response + second_part
        elif body.find('GetVideoSources') > 0:
            response = get_response(service, Onvif, body)
            response = replace_token(response, needed_ip)
            if int(cam_num) < 0:
                first_part = response[:response.rfind('GetVideoSourcesResponse')]
                first_part = first_part[:first_part.rfind('>') + 1]
                second_part = response[response.rfind('VideoSources>'):]
                second_part = second_part[second_part.find('<'):]
                response = first_part
                for i in range(1, len(ips)):
                    Onvif = make_request(cameras[paradigm][auditory][ips[i]]['ip'], cameras[paradigm][auditory][ips[i]]['port'])

                    add_response = get_response(service, Onvif, body, ips[i])
                    add_response = replace_token(add_response, ips[i])
                    add_response = add_response[add_response.find('GetVideoSourcesResponse') + 24:add_response.rfind('GetVideoSourcesResponse')]
                    add_response = add_response[:add_response.rfind('>') + 1]

                    response = response + add_response
                response = response + second_part
                response = re.sub("^\s+|\n|\r|\s+$", '', response)

        elif (body.rfind('Token') - 8) != body.rfind('UsernameToken') and body.rfind('Token') > -1 and body.rfind('UsernameToken') > -1:
            token_with_ip = body[body.rfind('Token>', 0, body.rfind('Token>') - 7) + 6:]
            token_with_ip = token_with_ip[: token_with_ip.find('<')]
            octet = token_with_ip.split('_')[0]

            token_without_ip = token_with_ip[len(octet) + 1:]
            body = body.replace(token_with_ip, token_without_ip)
            if body.find('GetVideoEncoderConfigurationOptions') > 0 or body.find('GetMetadataConfigurationOptions') > 0:
                if body.find('ConfigurationToken>') > 0:
                    token_with_ip = body[body.find('ConfigurationToken>') + 19:]
                    token_with_ip = token_with_ip[: token_with_ip.find('<')]
                    token_without_ip = token_with_ip[len(octet) + 1:]
                    body = body.replace(token_with_ip, token_without_ip)

            Onvif = make_request(cameras[paradigm][auditory][octet]['ip'], cameras[paradigm][auditory][octet]['port'])

            response = get_response(service, Onvif, body)

            if body.find('GetStreamUri') > 0:
                old_streamuri = response[response.find('>rtsp') + 1:]
                old_streamuri = old_streamuri[: old_streamuri.find('<')]
                streamuri = cameras[paradigm][auditory][octet]['streamuri']
                response = response.replace(old_streamuri, streamuri)
            response = replace_token(response, octet)

            if body.find('GetSnapshotUri'):
                if response.find('255.255.255.255:80'):
                    response = replace_all(response, get_replace_template(ipport, cameras[paradigm][auditory][octet]['ip']))
        else:
            response = get_response(service, Onvif, body)
            response = replace_token(response, ips[0])
        response = re.sub("^\s+|\n|\r|\s+$", '', response)

    return return_soap(response)
